﻿using Excavator.Drawnings;
using Excavator.Exceptions;
using System.Text;
using static System.Windows.Forms.LinkLabel;
namespace Excavator.CollectionGenericObjects;

/// <summary>
/// Класс-хранилище коллекций
/// </summary>
/// <typeparam name="T"></typeparam>
public class StorageCollection<T>
    where T : DrawningTrackedVehicle
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<string, ICollectionGenericObjects<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        if (string.IsNullOrEmpty(name)) return;
        if (_storages.ContainsKey(name)) return;
        if (collectionType == CollectionType.None) return;
        if (collectionType == CollectionType.Massive)
        {
            _storages[name] = new MassiveGenericObjects<T>();
        }
        else if (collectionType == CollectionType.List)
        {
            _storages[name] = new ListGenericObjects<T>();
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name"></param>
    public void DelCollection(string name)
    {
        if (name == null || !_storages.ContainsKey(name))
        {
            return;
        }

        _storages.Remove(name);
    }

    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            if (name == null || !_storages.ContainsKey(name)) { return null; }
            return _storages[name];
            
        }
    }
    /// <summary>
    /// Ключевое слово, с которого должен начинаться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionsStorage";
    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";
    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";

    /// <summary>
    /// Сохранение информации по автомобилям в хранилище в файл
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    public void SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            throw new NullReferenceException("В хранилище отсутствуют коллекции для сохранения");
        }

        if (File.Exists(filename))
        {
            File.Delete(filename);
        }

        using (StreamWriter writer = new StreamWriter(filename))
        {
            writer.Write(_collectionKey);
            foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
            {
                writer.Write(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0)
                {
                    continue;
                }

                writer.Write(value.Key);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.GetCollectionType);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.MaxCount);
                writer.Write(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;
                    if (string.IsNullOrEmpty(data))
                    {
                        continue;
                    }

                    writer.Write(data);
                    writer.Write(_separatorItems);
                }
            }
        }

    }
    /// <summary>
    /// Загрузка информации по автомобилям в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new FileNotFoundException("Файл не существует");
        }

        using (StreamReader reader = File.OpenText(filename))
        {
            string? str = reader.ReadLine();
            if (str == null || str.Length == 0)
            {
                throw new FileFormatException("В файле нет данных");
            }
            if (!str.Equals(_collectionKey))
            {
                throw new FileFormatException("В файле неверные данные");
            }
            _storages.Clear();
            string? strs = "";

            while ((strs = reader.ReadLine()) != null)
            {
                string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }

                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);

                if (collection == null)
                {
                    throw new InvalidOperationException("Не удалось создать коллекцию");
                }
                collection.MaxCount = Convert.ToInt32(record[2]);
                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);

                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningTrackedVehicle() is T ship)
                    {
                        try
                        {
                            if (collection.Insert(ship) == -1)
                            {
                                throw new InvalidOperationException("Объект не удалось добавить в коллекцию: " + record[3]);
                            }
                        }
                        catch (CollectionOverflowException ex)
                        {
                            throw new OverflowException("Коллекция переполнена", ex);
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }

        }
    }
    /// <summary>
	/// Создание коллекции по типу
	/// </summary>
	/// <param name="collectionType"></param>
	/// <returns></returns>
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}
