﻿using Excavator.CollectionGenericObjects;
using Excavator.Drawnings;
using Excavator.Exceptions;
using Microsoft.Extensions.Logging;

namespace Excavator;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormTrackedVehicleCollection : Form
{
    /// <summary>
	/// Хранилише коллекций
	/// </summary>
	private readonly StorageCollection<DrawningTrackedVehicle> _storageCollection;
    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Логер
    /// </summary>
    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTrackedVehicleCollection(ILogger<FormTrackedVehicleCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;

    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    /// <summary>
    /// Добавление автомобиля
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddCar_Click(object sender, EventArgs e)
    {
        FormTrackedVehicleConfig form = new();
        form.Show();
        form.AddEvent(SetCar);
    }

    /// <summary>
    /// Добавление автомобиля в коллекцию
    /// </summary>
    /// <param name="car"></param>
    private void SetCar(DrawningTrackedVehicle? car)
    {
        try
        {
            if (_company == null || car == null)
            {
                return;
            }

            if (_company + car != -1)
            {
                MessageBox.Show("Объект добавлен");
                _logger.LogInformation($"Добавлен объект {car.GetDataForSave()}");
                pictureBox.Image = _company.Show();
            }
        }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogWarning($"Ошибка: {ex.Message}");
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonDelExcavator_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            int pos = Convert.ToInt32(maskedTextBox.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                _logger.LogInformation($"Удален объект по позиции {pos}");
                pictureBox.Image = _company.Show();
            }
        }
        catch (ObjectNotFoundException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError($"Ошибка: {ex.Message}");
        }
        catch (PositionOutOfCollectionException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError($"Ошибка: {ex.Message}");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningTrackedVehicle? trackedVehicle = null;
        int counter = 100;
        while (trackedVehicle == null)
        {
            trackedVehicle = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        if (trackedVehicle == null)
        {
            return;
        }

        FormExcavator form = new()
        {
            SetTrackedVehicle = trackedVehicle
        };
        form.ShowDialog();
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }
    /// <summary>
	/// Добавление коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
      private void buttonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: Заполнены не все данные для добавления коллекции");
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            _logger.LogInformation($"Добавлена коллекция: {textBoxCollectionName.Text} типа: {collectionType}");
            RerfreshListBoxItems();
        }
    /// <summary>
	/// Удаление коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        _logger.LogInformation($"Удалена коллекция: {listBoxCollection.SelectedItem.ToString()}");

        RerfreshListBoxItems();
    }
    /// <summary>
	/// Обновление списка в listBoxCollection
	/// </summary>
	private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }

    }
    /// <summary>
	/// Создание компании
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        ICollectionGenericObjects<DrawningTrackedVehicle>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new Garage(pictureBox.Width, pictureBox.Height, collection);
                break;
        }

        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Обработка нажатия "Сохранение"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }
    /// <summary>
	/// Обработка нажатия "Загрузка"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RerfreshListBoxItems();
                _logger.LogInformation("Загрузка из фала: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

 
}

